from enum import Enum

HEADERS = {
    "Accept": (
        "text/html,"
        "application/xhtml+xml,"
        "application/xml;q=0.9,"
        "image/avif,"
        "image/webp,"
        "image/apng,"
        "*/*;q=0.8,"
        "application/signed-exchange;v=b3;q=0.7"
    ),
    "Accept-Encoding": "gzip, deflate",
    "Accept-Language": "en-US,en;q=0.9",
    "User-Agent": (
        "Mozilla/5.0 (Windows NT 10.0; Win64; x64) "
        "AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/126.0.0.0 "
        "Safari/537.36"
    ),
    "x-requested-with": "XMLHttpRequest",
}


class Theme(int, Enum):
    CHARACTERS = 1
    OBJECTS = 2
    ANIMALS = 14


class Answer(int, Enum):
    YES = 0
    NO = 1
    I_DONT_KNOW = 2
    PROBABLY = 3
    PROBABLY_NOT = 4


THEMES = {
    "en": [Theme.CHARACTERS, Theme.OBJECTS, Theme.ANIMALS],
    "ar": [Theme.CHARACTERS],
    "cn": [Theme.CHARACTERS],
    "de": [Theme.CHARACTERS, Theme.ANIMALS],
    "es": [Theme.CHARACTERS, Theme.ANIMALS],
    "fr": [Theme.CHARACTERS, Theme.OBJECTS, Theme.ANIMALS],
    "il": [Theme.CHARACTERS],
    "it": [Theme.CHARACTERS, Theme.ANIMALS],
    "jp": [Theme.CHARACTERS, Theme.ANIMALS],
    "kr": [Theme.CHARACTERS],
    "nl": [Theme.CHARACTERS],
    "pl": [Theme.CHARACTERS],
    "pt": [Theme.CHARACTERS],
    "ru": [Theme.CHARACTERS],
    "tr": [Theme.CHARACTERS],
    "id": [Theme.CHARACTERS],
}



import re
from contextlib import asynccontextmanager, suppress
from dataclasses import dataclass
from typing import Any, Dict, List, Optional, cast

from cookit.pyd import type_validate_json
from httpx import AsyncClient, Cookies
from pydantic import BaseModel, ValidationError

from .const import HEADERS, THEMES, Answer, Theme
from .errors import CanNotGoBackError, GameEndedError


@dataclass
class GameState:
    session: str
    signature: str

    question: str
    akitude: str = "defi.png"
    progression: float = 0.0
    step: int = 0
    win: bool = False
    step_last_proposition: Optional[int] = None
    ended: bool = False


class AnswerResp(BaseModel):
    akitude: str
    step: int
    progression: float
    question_id: int
    question: str
    completion: Optional[str] = None

    @property
    def akitude_url(self) -> str:
        return Akinator.get_akitude_url(self.akitude)


class WinResp(BaseModel):
    completion: str
    id_proposition: int
    id_base_proposition: int
    valide_contrainte: int
    name_proposition: str
    description_proposition: str
    flag_photo: int
    photo: str
    pseudo: str
    nb_elements: int


class Akinator:
    def __init__(
        self,
        lang: str = "cn",
        theme: Optional[Theme] = None,
        child_mode: bool = False,
        **cli_kwargs,
    ) -> None:
        if lang not in THEMES:
            raise ValueError(f"Unsupported language: {lang}")

        if not theme:
            theme = THEMES[lang][0]
        elif theme not in THEMES[lang]:
            raise ValueError(f"Unsupported theme: {theme}")

        self.lang: str = lang
        self.theme: Theme = theme
        self.child_mode: bool = child_mode

        self._state: Optional[GameState] = None

        self.cli_kwargs: Dict[str, Any] = cli_kwargs
        self.cookies: Optional[Cookies] = None

    @property
    def state(self) -> GameState:
        if not self._state:
            raise ValueError("Game not started")
        return self._state

    @property
    def base_url(self) -> str:
        return f"https://{self.lang}.akinator.com"

    @asynccontextmanager
    async def create_client(self):
        async with AsyncClient(  # noqa: S113
            base_url=self.base_url,
            headers=HEADERS,
            follow_redirects=True,
            cookies=self.cookies,
            **self.cli_kwargs,
        ) as cli:
            try:
                yield cli
            finally:
                self.cookies = cli.cookies

    @staticmethod
    def get_akitude_url(akitude: str) -> str:
        return f"https://cn.akinator.com/assets/img/akitudes_670x1096/{akitude}"

    async def get_akitude_image(self, akitude: str) -> bytes:
        async with self.create_client() as cli:
            return (
                (await cli.get(self.get_akitude_url(akitude)))
                .raise_for_status()
                .content
            )

    def make_answer_req_data(self):
        state = self.state
        return {
            "step": state.step,
            "progression": state.progression,
            "sid": self.theme.value,
            "cm": self.child_mode,
            "session": state.session,
            "signature": state.signature,
        }

    def handle_answer_resp(self, resp: AnswerResp):
        state = self.state
        if state.win:
            state.win = False
        state.step = resp.step
        state.progression = resp.progression
        state.question = resp.question
        state.akitude = resp.akitude

    def handle_win_resp(self, _: WinResp):
        state = self.state
        state.win = True
        state.step_last_proposition = state.step

    def ensure_not_end(self):
        state = self.state
        if state.ended:
            raise GameEndedError

    def ensure_not_win(self):
        state = self.state
        self.ensure_not_end()
        if state.win:
            raise RuntimeError(
                "Game already win, "
                "if you want to continue, please call `continue_answer`.",
            )

    def ensure_win(self):
        state = self.state
        self.ensure_not_end()
        if not state.win:
            raise RuntimeError("Game not win")

    def ensure_can_back(self):
        state = self.state
        self.ensure_not_win()
        if state.step <= 0:
            raise CanNotGoBackError

    async def start(self):
        url = "/game"
        data = {"sid": self.theme.value, "cm": self.child_mode}
        async with self.create_client() as cli:
            resp_text = (await cli.post(url, data=data)).raise_for_status().text

        session, signature = cast(
            List[str],
            re.findall(r"[a-zA-Z0-9+/]+==", resp_text),
        )[-2:]

        question_match = re.search(
            (
                r'<div class="bubble-body"><p class="question-text" id="question-label">'
                r"(?P<question>.+?)"
                r"</p></div>"
            ),
            resp_text,
        )
        if not question_match:
            raise ValueError("Failed to find question")
        question: str = question_match["question"]

        self._state = GameState(session, signature, question)
        return self._state

    async def answer(self, answer: Answer):
        state = self.state
        self.ensure_not_win()

        url = "/answer"
        data = {
            **self.make_answer_req_data(),
            "answer": answer.value,
            "step_last_proposition": (x if (x := state.step_last_proposition) else ""),
        }
        async with self.create_client() as cli:
            resp_text = ((await cli.post(url, data=data)).raise_for_status()).text

        with suppress(ValidationError):
            self.handle_answer_resp(resp := type_validate_json(AnswerResp, resp_text))
            return resp

        with suppress(ValidationError):
            self.handle_win_resp(resp := type_validate_json(WinResp, resp_text))
            return resp

        state.ended = True
        state.akitude = "deception.png"
        raise GameEndedError

    async def continue_answer(self):
        self.ensure_win()

        url = "/exclude"
        data = self.make_answer_req_data()
        async with self.create_client() as cli:
            resp_text = ((await cli.post(url, data=data)).raise_for_status()).text

        self.handle_answer_resp(resp := type_validate_json(AnswerResp, resp_text))
        return resp

    async def back(self):
        self.ensure_can_back()

        url = "/cancel_answer"
        data = self.make_answer_req_data()
        async with self.create_client() as cli:
            resp_text = ((await cli.post(url, data=data)).raise_for_status()).text

        self.handle_answer_resp(resp := type_validate_json(AnswerResp, resp_text))
        return resp



import asyncio
import sys
from contextlib import suppress

from . import (
    THEMES,
    Akinator,
    Answer,
    CanNotGoBackError,
    GameEndedError,
    Theme,
    WinResp,
    __version__,
)


async def main() -> int:
    print(f"CooAki v{__version__} | Console Demo")
    print()

    lang = (
        input(
            f"Available languages: {', '.join(THEMES)}\n"
            f'Input language (Defaults to "cn"): ',
        ).lower()
        or "cn"
    )
    if lang not in THEMES:
        print("Invalid language")
        return 1

    themes = THEMES[lang]
    if len(themes) == 1:
        theme = themes[0]
    else:
        default_theme = themes[0]
        theme = (
            input(
                f"Available themes: "
                f"{', '.join(f'{x} ({x.name.capitalize()})' for x in themes)}\n"
                f"Input theme (Defaults to {default_theme}): ",
            ).lower()
            or default_theme
        )
        try:
            theme = Theme(int(theme))
            assert theme in themes
        except Exception:
            print("Invalid theme")
            return 1

    child_mode = (input("Enable child mode? (y/N) ").lower() or "n") == "y"

    print()
    print(f"Using language: {lang}")
    print(f"Using theme: {theme.name.capitalize()}")
    print(f"Child mode {'enabled' if child_mode else 'disabled'}")
    print()

    aki = Akinator(lang, theme, child_mode)
    await aki.start()

    while not aki.state.ended:
        answer_tip = ", ".join(
            f"{x + 1} ({x.name.capitalize().replace('_', ' ')})" for x in Answer
        )

        msg = input(
            f"{aki.state.step + 1}: {aki.state.question}\n"
            f"Answer: {answer_tip}, B (Back), Ctrl-C (Quit)\n"
            f"Input answer: ",
        ).lower()
        print()

        if msg.isdigit():
            try:
                answer = Answer(int(msg) - 1)
            except Exception:
                print("Invalid answer")
                continue

            try:
                resp = await aki.answer(answer)
            except GameEndedError:
                print("You beat me!")
                break

            if isinstance(resp, WinResp):
                should_continue = (
                    input(
                        f"I guess: {resp.name_proposition} - {resp.description_proposition}\n"
                        f"Photo URL: {resp.photo} (From: {resp.pseudo})\n"
                        f"Continue? (y/N) ",
                    ).lower()
                    or "n"
                ) == "y"
                if not should_continue:
                    break
                print()
                await aki.continue_answer()
                continue

        elif msg == "b":
            try:
                await aki.back()
            except CanNotGoBackError:
                print("Cant go back any further!")
                print()

        else:
            print("Invalid answer")
            print()

    return 0


with suppress(KeyboardInterrupt):
    sys.exit(asyncio.run(main()))



from .akinator import (
    Akinator as Akinator,
    AnswerResp as AnswerResp,
    GameState as GameState,
    WinResp as WinResp,
)
from .const import THEMES as THEMES, Answer as Answer, Theme as Theme
from .errors import (
    CanNotGoBackError as CanNotGoBackError,
    GameEndedError as GameEndedError,
)

__version__ = "0.2.0"
